#pragma once

#include "factor_node.h"

#include <util/generic/vector.h>

namespace NSegmentator {

class TFactorTree;
class TBlockStats;

float DivideOrNaN(float a, float b);

template <typename T, typename TAcc>
float CalcAvg(const TVector<T>& elems, TAcc init);

template <typename T>
float CalcAvg(const TVector<T>& elems);

template <typename T>
float CalcMiddle(const TVector<T>& elems);

template <typename T>
float CalcMax(const TVector<T>& elems);

template <typename T>
float CalcAmplitude(const TVector<T>& elems);

template <typename T>
float CalcVar(const TVector<T>& elems);


void CalcMetaFactors(TFactorTree& factorTree);
void CalcCharFactors(TFactors& factors, TFactorNode* factorNode, TFactorTree& factorTree);
void CalcTreeFactors(TFactors& factors, TFactorNode* factorNode, TFactorTree& factorTree);


enum ECommonTreeFactors {
    CTF_FACTOR_0 = 0,
    CTF_FACTOR_1,
    CTF_FACTOR_2,
    CTF_FACTOR_3,
    CTF_FACTOR_4,
    CTF_FACTOR_5,
    CTF_FACTOR_6,
    CTF_FACTOR_7,
    CTF_FACTOR_8,
    CTF_FACTOR_9,
    CTF_FACTOR_10,
    CTF_FACTOR_11,
    CTF_FACTOR_12,
    CTF_FACTOR_13,
    CTF_FACTOR_14,
    CTF_FACTOR_15,
    CTF_FACTOR_16,
    CTF_FACTOR_17,
    CTF_FACTOR_18,
    CTF_FACTOR_19,
    CTF_FACTOR_20,
    CTF_FACTOR_21,
    CTF_FACTOR_22,
    CTF_FACTOR_23,
    CTF_FACTOR_24,
    CTF_FACTOR_25,
    CTF_FACTOR_26,
    CTF_FACTOR_27,
    CTF_FACTOR_28,
    CTF_FACTOR_29,
    CTF_FACTOR_30,
    CTF_FACTOR_31,
    CTF_FACTOR_32,
    CTF_FACTOR_33,
    CTF_FACTOR_34,
    CTF_FACTOR_35,
    CTF_FACTOR_36,
    CTF_FACTOR_37,
    CTF_FACTOR_38,
    CTF_FACTOR_39,
    CTF_FACTOR_40,
    CTF_FACTOR_41,
    CTF_FACTOR_42,
    CTF_FACTOR_43,
    CTF_FACTOR_44,
    CTF_FACTOR_45,
    CTF_FACTOR_46,
    CTF_FACTOR_47,
    CTF_FACTOR_48,
    CTF_FACTOR_49,
    CTF_FACTOR_50,
    CTF_FACTOR_51,
    CTF_FACTOR_52,
    CTF_FACTOR_53,
    CTF_FACTOR_54,
    CTF_FACTOR_55,
    CTF_FACTOR_56,
    CTF_FACTOR_57,
    CTF_FACTOR_58,
    CTF_FACTOR_59,
    CTF_FACTOR_60,
    CTF_FACTOR_61,
    CTF_FACTOR_62,
    CTF_FACTOR_63,
    CTF_FACTOR_64,
    CTF_FACTOR_65,
    CTF_FACTOR_66,
    CTF_FACTOR_67,
    CTF_FACTOR_68,
    CTF_FACTOR_69,
    CTF_FACTOR_70,
    CTF_FACTOR_71,
    CTF_FACTOR_72,
    CTF_FACTOR_73,
    CTF_FACTOR_74,
    CTF_FACTOR_75,
    CTF_FACTOR_76,
    CTF_FACTOR_77,
    CTF_FACTOR_78,
    CTF_FACTOR_79,
    CTF_FACTOR_80,
    CTF_FACTOR_81,
    CTF_FACTOR_82,
    CTF_FACTOR_83,
    CTF_FACTOR_84,
    CTF_FACTOR_85,
    CTF_FACTOR_86,
    CTF_FACTOR_87,
    CTF_FACTOR_88,
    CTF_FACTOR_89,
    CTF_FACTOR_90,
    CTF_FACTOR_91,
    CTF_FACTOR_92,
    CTF_FACTOR_93,
    CTF_FACTOR_94,
    CTF_FACTOR_95,
    CTF_FACTOR_96,
    CTF_FACTOR_97,
    CTF_FACTOR_98,
    CTF_FACTOR_99,
    CTF_FACTOR_100,
    CTF_FACTOR_101,
    CTF_FACTOR_102,
    CTF_FACTOR_103,
    CTF_FACTOR_104,
    CTF_FACTOR_105,
    CTF_FACTOR_106,
    CTF_FACTOR_107,
    CTF_FACTOR_108,
    CTF_FACTOR_109,
    CTF_FACTOR_110,
    CTF_FACTOR_111,
    CTF_FACTOR_112,
    CTF_FACTOR_113,
    CTF_FACTOR_114,
    CTF_FACTOR_115,
    CTF_FACTOR_116,
    CTF_FACTOR_117,
    CTF_FACTOR_118,
    CTF_FACTOR_119,
    CTF_FACTOR_120,
    CTF_FACTOR_121,
    CTF_FACTOR_122,
    CTF_FACTOR_123,
    CTF_FACTOR_124,
    CTF_FACTOR_125,
    CTF_FACTOR_126,
    CTF_FACTOR_127,
    CTF_FACTOR_128,
    CTF_FACTOR_129,
    CTF_FACTOR_130,
    CTF_FACTOR_131,
    CTF_FACTOR_132,
    CTF_FACTOR_133,
    CTF_FACTOR_134,
    CTF_FACTOR_135,
    CTF_FACTOR_136,
    CTF_FACTOR_137,
    CTF_FACTOR_138,
    CTF_FACTOR_139,
    CTF_FACTOR_140,
    CTF_FACTOR_141,
    CTF_FACTOR_142,
    CTF_FACTOR_143,
    CTF_FACTOR_144,
    CTF_FACTOR_145,
    CTF_FACTOR_146,
    CTF_FACTOR_147,
    CTF_FACTOR_148,
    CTF_FACTOR_149,
    CTF_FACTOR_150,
    CTF_FACTOR_151,
    CTF_FACTOR_152,
    CTF_FACTOR_153,
    CTF_FACTOR_154,
    CTF_FACTOR_155,
    CTF_FACTOR_156,
    CTF_FACTOR_157,
    CTF_FACTOR_158,
    CTF_FACTOR_159,
    CTF_FACTOR_160,
    CTF_FACTOR_161,
    CTF_FACTOR_162,
    CTF_FACTOR_163,
    CTF_FACTOR_164,
    CTF_FACTOR_165,
    CTF_FACTOR_166,
    CTF_FACTOR_167,
    CTF_FACTOR_168,
    CTF_FACTOR_169,
    CTF_FACTOR_170,
    CTF_FACTOR_171,
    CTF_FACTOR_172,
    CTF_FACTOR_173,
    CTF_FACTOR_174,
    CTF_FACTOR_175,
    CTF_FACTOR_176,
    CTF_FACTOR_177,
    CTF_FACTOR_178,
    CTF_FACTOR_179,
    CTF_FACTOR_180,
    CTF_FACTOR_181,
    CTF_FACTOR_182,
    CTF_FACTOR_183,
    CTF_FACTOR_184,
    CTF_FACTOR_185,
    CTF_FACTOR_186,
    CTF_FACTOR_187,
    CTF_FACTOR_188,
    CTF_FACTOR_189,
    CTF_FACTOR_190,
    CTF_FACTOR_191,
    CTF_FACTOR_192,
    CTF_FACTOR_193,
    CTF_FACTOR_194,
    CTF_FACTOR_195,
    CTF_FACTOR_196,
    CTF_FACTOR_197,
    CTF_FACTOR_198,
    CTF_FACTOR_199,
    CTF_FACTOR_200,
    CTF_FACTOR_201,
    CTF_FACTOR_202,
    CTF_FACTOR_203,
    CTF_FACTOR_204,
    CTF_FACTOR_205,
    CTF_FACTOR_206,
    CTF_FACTOR_207,
    CTF_FACTOR_208,
    CTF_FACTOR_209,
    CTF_FACTOR_210,
    CTF_FACTOR_211,
    CTF_FACTOR_212,
    CTF_FACTOR_213,
    CTF_FACTOR_214,
    CTF_FACTOR_215,
    CTF_FACTOR_216,
    CTF_FACTOR_217,
    CTF_FACTOR_218,
    CTF_FACTOR_219,
    CTF_FACTOR_220,
    CTF_FACTOR_221,
    CTF_FACTOR_222,
    CTF_FACTOR_223,
    CTF_FACTOR_224,
    CTF_FACTOR_225,
    CTF_FACTOR_226,
    CTF_FACTOR_227,
    CTF_FACTOR_228,
    CTF_FACTOR_229,
    CTF_FACTOR_230,
    CTF_FACTOR_231,
    CTF_FACTOR_232,
    CTF_FACTOR_233,
    CTF_FACTOR_234,
    CTF_FACTOR_235,
    CTF_FACTOR_236,
    CTF_FACTOR_237,
    CTF_FACTOR_238,
    CTF_FACTOR_239,
    CTF_FACTOR_240,
    CTF_FACTOR_241,
    CTF_FACTOR_242,
    CTF_FACTOR_243,
    CTF_FACTOR_244,
    CTF_FACTOR_245,
    CTF_FACTOR_246,
    CTF_FACTOR_247,
    CTF_FACTOR_248,
    CTF_FACTOR_249,
    CTF_FACTOR_250,
    CTF_FACTOR_251,
    CTF_FACTOR_252,
    CTF_FACTOR_253,
    CTF_FACTOR_254,
    CTF_FACTOR_255,
    CTF_FACTOR_256,
    CTF_FACTOR_257,
    CTF_FACTOR_258,
    CTF_FACTOR_259,
    CTF_FACTOR_260,
    CTF_FACTOR_261,
    CTF_FACTOR_262,
    CTF_FACTOR_263,
    CTF_FACTOR_264,
    CTF_FACTOR_265,
    CTF_FACTOR_266,
    CTF_FACTOR_267,
    CTF_FACTOR_268,
    CTF_FACTOR_269,
    CTF_FACTOR_270,
    CTF_FACTOR_271,
    CTF_FACTOR_272,
    CTF_FACTOR_273,
    CTF_FACTOR_274,
    CTF_FACTOR_275,
    CTF_FACTOR_276,
    CTF_FACTOR_277,
    CTF_FACTOR_278,
    CTF_FACTOR_279,
    CTF_FACTOR_280,
    CTF_FACTOR_281,
    CTF_FACTOR_282,
    CTF_FACTOR_283,
    CTF_FACTOR_284,
    CTF_FACTOR_285,
    CTF_FACTOR_286,
    CTF_FACTOR_287,
    CTF_FACTOR_288,
    CTF_FACTOR_289,
    CTF_FACTOR_290,
    CTF_FACTOR_291,
    CTF_FACTOR_292,
    CTF_FACTOR_293,
    CTF_FACTOR_294,
    CTF_FACTOR_295,
    CTF_FACTOR_296,
    CTF_FACTOR_297,
    CTF_FACTOR_298,
    CTF_FACTOR_299,
    CTF_FACTOR_300,
    CTF_FACTOR_301,
    CTF_FACTOR_302,
    CTF_FACTOR_303,
    CTF_FACTOR_304,
    CTF_FACTOR_305,
    CTF_FACTOR_306,
    CTF_FACTOR_307,
    CTF_FACTOR_308,
    CTF_FACTOR_309,
    CTF_FACTOR_310,
    CTF_FACTOR_311,
    CTF_FACTOR_312,
    CTF_FACTOR_313,
    CTF_FACTOR_314,
    CTF_FACTOR_315,
    CTF_FACTOR_316,
    CTF_FACTOR_317,
    CTF_FACTOR_318,
    CTF_FACTOR_319,
    CTF_FACTOR_320,
    CTF_FACTOR_321,
    CTF_FACTOR_322,
    CTF_FACTOR_323,
    CTF_FACTOR_324,
    CTF_FACTOR_325,
    CTF_FACTOR_326,
    CTF_FACTOR_327,
    CTF_FACTOR_328,
    CTF_FACTOR_329,
    CTF_FACTOR_330,
    CTF_FACTOR_331,

    CTF_COUNT
};


enum ECommonCharFactors {
    CCF_FACTOR_0 = 0,
    CCF_FACTOR_1,
    CCF_FACTOR_2,
    CCF_FACTOR_3,
    CCF_FACTOR_4,
    CCF_FACTOR_5,
    CCF_FACTOR_6,
    CCF_FACTOR_7,
    CCF_FACTOR_8,
    CCF_FACTOR_9,
    CCF_FACTOR_10,
    CCF_FACTOR_11,
    CCF_FACTOR_12,
    CCF_FACTOR_13,
    CCF_FACTOR_14,
    CCF_FACTOR_15,
    CCF_FACTOR_16,
    CCF_FACTOR_17,
    CCF_FACTOR_18,
    CCF_FACTOR_19,
    CCF_FACTOR_20,
    CCF_FACTOR_21,
    CCF_FACTOR_22,
    CCF_FACTOR_23,
    CCF_FACTOR_24,
    CCF_FACTOR_25,
    CCF_FACTOR_26,
    CCF_FACTOR_27,
    CCF_FACTOR_28,
    CCF_FACTOR_29,
    CCF_FACTOR_30,
    CCF_FACTOR_31,
    CCF_FACTOR_32,
    CCF_FACTOR_33,
    CCF_FACTOR_34,
    CCF_FACTOR_35,
    CCF_FACTOR_36,
    CCF_FACTOR_37,
    CCF_FACTOR_38,
    CCF_FACTOR_39,
    CCF_FACTOR_40,
    CCF_FACTOR_41,
    CCF_FACTOR_42,
    CCF_FACTOR_43,
    CCF_FACTOR_44,
    CCF_FACTOR_45,
    CCF_FACTOR_46,
    CCF_FACTOR_47,
    CCF_FACTOR_48,
    CCF_FACTOR_49,
    CCF_FACTOR_50,
    CCF_FACTOR_51,
    CCF_FACTOR_52,
    CCF_FACTOR_53,
    CCF_FACTOR_54,
    CCF_FACTOR_55,
    CCF_FACTOR_56,
    CCF_FACTOR_57,
    CCF_FACTOR_58,
    CCF_FACTOR_59,
    CCF_FACTOR_60,
    CCF_FACTOR_61,
    CCF_FACTOR_62,
    CCF_FACTOR_63,
    CCF_FACTOR_64,
    CCF_FACTOR_65,
    CCF_FACTOR_66,
    CCF_FACTOR_67,
    CCF_FACTOR_68,
    CCF_FACTOR_69,
    CCF_FACTOR_70,
    CCF_FACTOR_71,
    CCF_FACTOR_72,
    CCF_FACTOR_73,
    CCF_FACTOR_74,
    CCF_FACTOR_75,
    CCF_FACTOR_76,
    CCF_FACTOR_77,
    CCF_FACTOR_78,
    CCF_FACTOR_79,
    CCF_FACTOR_80,
    CCF_FACTOR_81,
    CCF_FACTOR_82,
    CCF_FACTOR_83,
    CCF_FACTOR_84,
    CCF_FACTOR_85,
    CCF_FACTOR_86,
    CCF_FACTOR_87,
    CCF_FACTOR_88,
    CCF_FACTOR_89,
    CCF_FACTOR_90,
    CCF_FACTOR_91,
    CCF_FACTOR_92,
    CCF_FACTOR_93,
    CCF_FACTOR_94,
    CCF_FACTOR_95,
    CCF_FACTOR_96,
    CCF_FACTOR_97,
    CCF_FACTOR_98,
    CCF_FACTOR_99,
    CCF_FACTOR_100,
    CCF_FACTOR_101,
    CCF_FACTOR_102,
    CCF_FACTOR_103,
    CCF_FACTOR_104,
    CCF_FACTOR_105,
    CCF_FACTOR_106,
    CCF_FACTOR_107,
    CCF_FACTOR_108,
    CCF_FACTOR_109,
    CCF_FACTOR_110,
    CCF_FACTOR_111,
    CCF_FACTOR_112,
    CCF_FACTOR_113,
    CCF_FACTOR_114,
    CCF_FACTOR_115,
    CCF_FACTOR_116,
    CCF_FACTOR_117,
    CCF_FACTOR_118,
    CCF_FACTOR_119,
    CCF_FACTOR_120,
    CCF_FACTOR_121,
    CCF_FACTOR_122,
    CCF_FACTOR_123,
    CCF_FACTOR_124,
    CCF_FACTOR_125,
    CCF_FACTOR_126,
    CCF_FACTOR_127,
    CCF_FACTOR_128,
    CCF_FACTOR_129,
    CCF_FACTOR_130,
    CCF_FACTOR_131,
    CCF_FACTOR_132,
    CCF_FACTOR_133,
    CCF_FACTOR_134,
    CCF_FACTOR_135,
    CCF_FACTOR_136,
    CCF_FACTOR_137,
    CCF_FACTOR_138,
    CCF_FACTOR_139,
    CCF_FACTOR_140,
    CCF_FACTOR_141,
    CCF_FACTOR_142,
    CCF_FACTOR_143,
    CCF_FACTOR_144,
    CCF_FACTOR_145,
    CCF_FACTOR_146,
    CCF_FACTOR_147,
    CCF_FACTOR_148,
    CCF_FACTOR_149,
    CCF_FACTOR_150,
    CCF_FACTOR_151,
    CCF_FACTOR_152,
    CCF_FACTOR_153,
    CCF_FACTOR_154,
    CCF_FACTOR_155,
    CCF_FACTOR_156,
    CCF_FACTOR_157,
    CCF_FACTOR_158,
    CCF_FACTOR_159,
    CCF_FACTOR_160,
    CCF_FACTOR_161,
    CCF_FACTOR_162,
    CCF_FACTOR_163,
    CCF_FACTOR_164,
    CCF_FACTOR_165,
    CCF_FACTOR_166,
    CCF_FACTOR_167,
    CCF_FACTOR_168,
    CCF_FACTOR_169,
    CCF_FACTOR_170,
    CCF_FACTOR_171,
    CCF_FACTOR_172,
    CCF_FACTOR_173,
    CCF_FACTOR_174,
    CCF_FACTOR_175,
    CCF_FACTOR_176,
    CCF_FACTOR_177,
    CCF_FACTOR_178,
    CCF_FACTOR_179,
    CCF_FACTOR_180,
    CCF_FACTOR_181,

    CCF_COUNT
};

}  // NSegmentator
